/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   SELOR - Servidor de Emails Livre Orientado a Regras                           
   Programado por Lucas Priori                                                   
   Versao: 1.2                                                                   
                                                                                 
   Linux - make linux                                                            
       or  make linux_nocap                                                      
   FreeBSD - gmake freebsd                                                       
                                                                                 
  *******************************************************************            
  Selor e um agente MTA para troca de emails via protocolo SMTP                 
  Copyright (C) 2012,2013,2014  Lucas Priori                                     
                                                                                 
  This program is free software; you can redistribute it and/or                  
  modify it under the terms of the GNU General Public License                    
  as published by the Free Software Foundation; either version 2                 
  of the License, or (at your option) any later version.                         
                                                                                 
  This program is distributed in the hope that it will be useful,                
  but WITHOUT ANY WARRANTY; without even the implied warranty of                 
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                  
  GNU General Public License for more details.                                   
                                                                                 
  You should have received a copy of the GNU General Public License              
  along with this program; if not, write to the Free Software                    
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<unistd.h>
#include<errno.h>
#include<dirent.h>
#include<signal.h>
#include<time.h>
#include<sys/stat.h>
#include<fcntl.h>
#include<pwd.h>
#include<grp.h>
#include<sys/socket.h>
#include<arpa/inet.h>
#include<dlfcn.h>
#include<sys/wait.h>
#include<netinet/in.h>
#include<resolv.h>
#include<netdb.h>
#include<pthread.h>
#ifndef FreeBSD
 #ifndef NoCAP
  #include<sys/prctl.h>
  #include<sys/capability.h>
 #endif
#endif



//-- Definicoes Globais --
#define _MAIN_
#include "inc/global.h"
#include "inc/selor_utils.h"
#include "inc/selor_dns.h"
#include "inc/selor_mod.h"
#include "inc/selor_rules.h"
#include "inc/selor_tls.h"

//-- Variaveis Globais-Locais --
pid_t qproc=0;

//-- Funcoes locais --
char proc_rcpt(char *ptr_email, char responder);
void rset(void);
void sinal_sigterm(int sinal);
void sinal_qlimit(int sinal);
void falha_interna(char finalizar);
void selor_setuid(uid_t local_uid);
void *mod_refresh(void *parametro);

//###############################################
//#            Programa principal               #
//###############################################
int main(int argc, char *argv[]){
 FILE *arq_conf=NULL, *arq_cid=NULL, *arq_lid=NULL;
 time_t temp_queue1, temp_queue2;
 pthread_t thmod_refresh;
 struct timespec;
 struct dirent *entr;
 struct stat dados_entr;
 struct cpl_struct arq_handler;
 struct str_dosd dosd_ctrl;
 struct str_pid_slot{
  DIR  *dir;
  char subdir[TAM_subdirs];
  unsigned int qtime;
  int *slots;
  int n_slots;
  int livres;
 }pid_slots[NUM_filas];
// -- SOCKET --
 struct sockaddr_in rede_local;
 //struct in_addr tmp_ip;
 int main_socket;
 int num_socket=1;
 // --
 char id_temp[TAM_id_fila];
 char cam_arq_conf[150]=CAM_arq_conf;
 char *dosd_buffer=NULL;
 char *param_p=NULL;
 char param_qp=0;
 char param_q=0;
 char param_s=0;
 char *ptr_smtp_cmd=NULL;
 char *ptr_aux1=NULL;
 char *ptr_aux2=NULL;
 char *ptr_aux3=NULL;
 char ret1=0;
 char manter_conectado=1;
 char sem_espaco=0;
 char cnx_int=0;
 char header_info_ini[3]={0, '(', 0};
 char header_info_fim[3]={0, '\n', 0};
 char header_info_auth[150];
 char header_info_tls[50];
 char df_alias=0;
 char mod_info_buffer[10];
 char mod_refresh_flag=0;
 int arq_lock;
 int auth_tolerance_cont1=0;
 int b_lidos=0;
 int arq_pid=0;
 int temp1=0;
 int temp2=0;
 int temp3=0;
 int temp4=0;
 int b_recebidos=0;
 int falta_receber=0;
 unsigned int *testa_fim_data=NULL;
 unsigned int *op_numerica=NULL;
 unsigned int t_restante=0;
 unsigned int cont1=0;
 unsigned int cont2=0;
 unsigned int cont3=0;
 unsigned int cont4=0;
 unsigned int cont5=0;
 unsigned int prox_bloco=0;
 unsigned int prox_bloco_broadcast=0;
 
 
 // -------------------------------------------------------
 // -- Lendo os parametros passados via linha de comando --
 if(argc > 1){
  for(cont1=1; cont1<argc; cont1++){
   if(!strcmp(argv[cont1], "-h") || !strcmp(argv[cont1], "--help")){
    Ajuda:
    printf("MTA SELOR v%s\n",VERSAO);
    printf("Option | Description\n");
    printf(" -p     Process a queue file by the ID number\n");
    printf(" -qp    Process a queue file at qlimit spool by the ID number\n");
    printf(" -q     Don't starts the process that handles the queue\n");
    printf(" -c     Indicates a configuration file\n");
    printf("        Default: %s\n",CAM_arq_conf);
    printf(" -s     Show settings\n\n");
    return 0;
   }else if(!strcmp(argv[cont1], "-q"))param_q=1;
   else if(!strcmp(argv[cont1], "-p")){
    if(argv[++cont1])param_p=argv[cont1];
    else goto Ajuda;
   }else if(!strcmp(argv[cont1], "-qp")){
    if(argv[++cont1])param_p=argv[cont1];
    else goto Ajuda;
    param_qp=1;
   }else if(!strcmp(argv[cont1], "-c")){
    if(argv[++cont1])selor_strncpy(cam_arq_conf, argv[cont1], sizeof(cam_arq_conf)-1);
    else goto Ajuda;
   }else if(!strcmp(argv[cont1], "-s")){
    param_s=1;
    printf("MTA SELOR v%s\n\n",VERSAO);
   }
  }
 }
 
 // --------- Zerando algumas variaveis --------------
 // Se eu zero na declaracao o compilador usa memset  
 // que e' perda desnecessaria de processamento       
 fpid[0]=0;
 id_fila[0]=0;
 smtp_cmd[0]=0;
 buffer[0]=0;
 linha[0]=0;
 hostname[0]=0;
 adomain[0]=0;
 auth_user[0]=0;
 auth_pass[0]=0;
 ult_dom_int[0]=0;
 ult_dom_ext[0]=0;
 id_pid[0]=0;
 id_temp[0]=0;
 header_info_auth[0]=0;
 header_info_tls[0]=0;
 memset(def_ops, 0, NUM_opcoes);
 memset(pid_slots, 0, sizeof(pid_slots));
 memset(&ips_locais, 0, sizeof(ips_locais));
 // -- Variaveis de regras --
 R_DATA_REPLY[0]=0;
 R_HADD[0]=0;

 // -- Lendo arquivo de configuracao --
 arq_conf=fopen(cam_arq_conf, "rb");
 if(!arq_conf){
  printf("%s not found\n", cam_arq_conf);
  return 1;
 }
 arq_handler.dest=linha;
 arq_handler.tambuffer=arq_handler.tamusado=TAM_buffer;
 while(!feof(arq_conf)){
  b_lidos=fread(&buffer, 1, TAM_buffer, arq_conf);
  buffer[b_lidos]=0;
  arq_handler.orig=buffer;
  while(!cpl(&arq_handler)){
   arq_handler.dest=linha;
   if(linha[0] && linha[0] != '#'){
    ptr_aux1=strchr(linha, ' ');
    if(!ptr_aux1)ptr_aux1=strchr(linha, 9);     // 9 -> TAB
    if(ptr_aux1){
     *ptr_aux1=0;
     ptr_aux1++;
     while((*ptr_aux1 == ' ' || *ptr_aux1 == 9) && *ptr_aux1)ptr_aux1++;
     if(*ptr_aux1 == '"' || *ptr_aux1 == '\''){
      ret1=*ptr_aux1;
      ptr_aux1++;
     }
     ptr_aux2=strchr(ptr_aux1, ret1);
     if(strlen(ptr_aux1) > TAM_defs){
      printf("ERROR: Parameter too big [%s]. Maximum size is %d\n-> '%s'\n", linha, TAM_defs, ptr_aux1);
      fclose(arq_conf);
      return 1;
     }
     if(ptr_aux2)*ptr_aux2=0;
     // -- Gravando informacao na opcao correta --
     temp1=def_parametro(linha, ptr_aux1);
     //if(param_s && temp1 == Module)printf("%s = \"%s\"\n", linha, DEF_module);
     if(temp1 < 0)printf("Warning: Unknown option and was ignored: %s [%s]\n", linha, ptr_aux1);
    }
   }
  }
 }
 fclose(arq_conf);
 
 if(param_s){
  for(cont1=0; cont1<NUM_opcoes; cont1++){
   if(def_ops_val[cont1] & OP_INT){
    op_numerica=ptr_op[cont1];
    if(cont1 == Tarpit_limit && DEF_tarpit_limit == 0xFFFFFFFF)printf("%s = 0 (Disabled)\n", nome_op[cont1]);
    else printf("%s = %u\n", nome_op[cont1], *op_numerica);
   }else {
    if(cont1 == Module)for(cont2=0; cont2<num_mods; cont2++)printf("Module = \"%s\"\n", (char *)DEF_module[cont2]);
    else printf("%s = \"%s\"\n", nome_op[cont1], (char *)ptr_op[cont1]);
   }
  }
  printf("\n");
  return 0;
 }
 
 // -- Resolvendo ips do Server_name --
 cont1=0;
 while((ips_locais[cont1]=resolver_dom(DEF_server_name, buffer, cont1)) && cont1 < MAX_ips_locais-1)cont1++;
 if(!cont1)printf("WRN: Could not resolve Server_name '%s'\n", DEF_server_name);
 
 // -- Checagens diversas --
 cont1=0;
 if(!DEF_module[0][0]){
  printf("You must specify a module in %s\n", cam_arq_conf);
  cont1=1;
 }
 pid_slots[0].dir=opendir(DEF_queue_path);
 if(!pid_slots[0].dir){
  printf("Not found [%s]\n", DEF_queue_path);
  cont1=1;
 }
 closedir(pid_slots[0].dir);
 pid_slots[0].dir=opendir(DEF_cp_errors_path);
 if(!pid_slots[0].dir){
  printf("Not found [%s]\n", DEF_cp_errors_path);
  cont1=1;
 }
 closedir(pid_slots[0].dir);
 pid_slots[0].dir=opendir(DEF_temp_path);
 if(!pid_slots[0].dir){
  printf("Not found [%s]\n", DEF_temp_path);
  cont1=1;
 }
 closedir(pid_slots[0].dir);
 // -- Checando pelo usurio e grupo
 pwd=getpwnam(DEF_user);
 grp=getgrnam(DEF_group);
 if(!pwd || !grp){
  printf("User(%s) or Group(%s) not found\n", DEF_user, DEF_group);
  cont1=1;
 }else {
  uid=pwd->pw_uid;
  gid=grp->gr_gid;
 }
 // -- Criando os arquivos de LOG
 umask(0027);
 if(DEF_central_log[0]){
  arq_pid=open(DEF_central_log, O_RDWR | O_CREAT, 0640);
  if(arq_pid < 0){
   printf("Could not create file : %s\nCheck directory permissions\n", DEF_central_log);
   cont1=1;
  }
  close(arq_pid);
  chown(DEF_central_log, uid, gid);
 }
 if(DEF_sumary_log[0]){
  arq_pid=open(DEF_sumary_log, O_RDWR | O_CREAT, 0640);
  if(arq_pid < 0){
   printf("Could not create file : %s\nCheck directory permissions\n", DEF_sumary_log);
   cont1=1;
  }
  close(arq_pid);
  chown(DEF_sumary_log, uid, gid);
 }
 if(DEF_warning_log[0]){
  arq_pid=open(DEF_warning_log, O_RDWR | O_CREAT, 0640);
  if(arq_pid < 0){
   printf("Could not create file : %s\nCheck directory permissions\n", DEF_warning_log);
   cont1=1;
  }
  close(arq_pid);
  chown(DEF_warning_log, uid, gid);
 }
 if(cont1){
  printf("Execution aborted.\n");
  return 1;
 }
 
 // ------------------------------------
 // Tarefas diversas de preparacao      
 selor_strncpy(hostname, DEF_server_name, sizeof(hostname)-1);
 if(!DEF_server_name[0])gethostname(hostname, sizeof(hostname)-1);
 if(!DEF_welcome_msg[0])selor_superstrncpy(DEF_welcome_msg, TAM_defs, "220 ", hostname, "(", DEF_sid, ") MTA Selor ", VERSAO, NULL);
 chdir(DEF_queue_path);     // Definindo diretorio padrao de trabalho
 arq_pid=open(DEF_pid_file, O_RDONLY, 0640);
 
 if(!param_p){
  // -----------------------------------------
  // Verificando se o servidor ja esta rodando
  if(arq_pid > 0){
   if(lockf(arq_pid, F_TEST, 0) < 0){
    temp1=read(arq_pid, buffer, TAM_buffer);
    buffer[temp1]=0;
    printf("MTA running. PID %s\n\n", buffer);
    close(arq_pid);
    return 1;
   }
   close(arq_pid);
   remove(DEF_pid_file);
  }
  
  
  // ---------------------------------
  //  Daemonizando-se                 
  // ---------------------------------
  if(!param_p){
   temp1=fork();
   if(temp1 < 0){
    printf("Fork fail\n");
    return 1;
   }
   if(temp1 > 0)return 0;   // O processo pai morre aqui
   setsid();                // Criando uma nova sessao de grupo para seus processos
  }
  // -- Fechando STDIN,STDOUT e STDERR --
  temp1=open("/dev/null", O_RDWR);             // Apontando stdin  para /dev/null
  close(0);
  close(1);
  close(2);
  dup2(temp1, 0);                              // Apontando stdin  para /dev/null
  dup2(temp1, 1);                              // Apontando stdout para /dev/null
  dup2(temp1, 2);                              // Apontando stderr para /dev/null
  close(temp1);
  // ----------------------
  // Criando arquivo de PID
  
  if(DEF_sid[0])selor_superstrncpy(DEF_pid_file, sizeof(DEF_pid_file), DEF_pid_file, ".", DEF_sid, NULL);
  arq_pid=open(DEF_pid_file, O_RDWR | O_CREAT, 0644);
  if(arq_pid < 0){
   selor_superstrncpy(buffer_log, TAM_buffer_log, "ERROR: Error creating file: ", DEF_pid_file, "\n", NULL);
   log_msg(DEF_warning_log);
   return 1;
  }
  if(lockf(arq_pid, F_TLOCK, 0) < 0){
   selor_superstrncpy(buffer_log, TAM_buffer_log, "ERROR: Server is running [", DEF_pid_file, "]\n", NULL);
   log_msg(DEF_warning_log);
   close(arq_pid);
   return 1;
  }
  ppid=getpid();
  temp1=sprintf(buffer, "%d", ppid);
  if(write(arq_pid, buffer, temp1) != temp1){
   selor_superstrncpy(buffer_log, TAM_buffer_log, "ERROR: Error writing file ", DEF_pid_file, "\n", NULL);
   log_msg(DEF_warning_log);
   close(arq_pid);
   return 1;
  }
 }
 // Ignorando alguns sinais
 signal(SIGCHLD, SIG_IGN);         // Para nao gerar ZUMBIS
 signal(SIGTSTP, SIG_IGN);
 signal(SIGTTOU, SIG_IGN);
 signal(SIGTTIN, SIG_IGN);
 signal(SIGPIPE, SIG_IGN);
 signal(SIGALRM, tempoesgotado);
 signal(SIGSEGV, capta_sinal);     // Capturando sinais SegmFAULT  
 signal(SIGHUP, capta_sinal);      // Capturando sinais SIGHUP     
 signal(SIGINT, capta_sinal);      // Capturando sinais SIGINT     
 signal(SIGQUIT, capta_sinal);     // Capturando sinais SIGQUIT    
 signal(SIGILL, capta_sinal);      // Capturando sinais SIGILL     
 signal(SIGABRT, capta_sinal);     // Capturando sinais SIGABRT    
 
 
 // ---------------------------------
 //  Carregando MODULOS              
 // ---------------------------------
 for(cont1=0; cont1<num_mods; cont1++){
  if(!(ptr_aux1=strchr(DEF_module[cont1], ' ')))ptr_aux1=strchr(DEF_module[cont1], 9);
  if(ptr_aux1){
   // Parametro existente
   ret1=*ptr_aux1;
   ptr_aux2=ptr_aux1;
   *ptr_aux1++=0;
   ptr_aux1=pular_carac(ptr_aux1, ret1);
   ptr_aux3=ptr_aux1;
  }else {
   // Parametro inexistente
   ptr_aux3=NULL;
   ret1=0;
  }
  
  temp1=0;
  setgid(gid);          //  Foi colocado aqui para o modulo iniciar com as permissao de grupo correta
  if((modulo[cont1]=dlopen(DEF_module[cont1], RTLD_NOW | RTLD_LOCAL))){
   mod_init[cont1]=dlsym(modulo[cont1], "mod_init");
   mod_dom_local[cont1]=dlsym(modulo[cont1], "mod_dom_local");
   mod_user_local[cont1]=dlsym(modulo[cont1], "mod_user_local");
   mod_ret_alias[cont1]=dlsym(modulo[cont1], "mod_ret_alias");
   mod_auth[cont1]=dlsym(modulo[cont1], "mod_auth");
   mod_test_quota[cont1]=dlsym(modulo[cont1], "mod_test_quota");
   mod_msg_error[cont1]=dlsym(modulo[cont1], "mod_msg_error");
   mod_shutdown[cont1]=dlsym(modulo[cont1], "mod_shutdown");
   mod_cnx_refresh[cont1]=dlsym(modulo[cont1], "mod_cnx_refresh");
   mod_ret_broadcast[cont1]=dlsym(modulo[cont1], "mod_ret_broadcast");
   if(mod_cnx_refresh[cont1])mod_refresh_flag=1;  // -> Modulo tem sistema de refresh
   strcpy(buffer_log, "Invalid module\n");
  }else snprintf(buffer_log, TAM_buffer_log, "dlerror: %s\n", dlerror());
  if(!mod_init[cont1] || !mod_dom_local[cont1] || !mod_user_local[cont1] ||
     !mod_ret_alias[cont1] || !mod_auth[cont1] || !mod_test_quota[cont1] ||
     !mod_msg_error[cont1] || !mod_shutdown[cont1] ||
     (temp1=mod_init[cont1](ptr_aux3, mod_id, mod_version, mod_info_buffer, sizeof(mod_info_buffer)))){
   if(temp1)snprintf(buffer_log, TAM_buffer_log, "MODULE Error: %s\n", mod_msg_error[cont1]());
   log_msg(DEF_warning_log);
   return 1;
  }
  // Procurando se o modulo ja esta registrado
  for(cont2=0; cont2<cont1; cont2++){
   if(!strcmp(mod_names[cont2].mod_id, mod_id)){
    // Modulo ja registrado
    snprintf(buffer_log, TAM_buffer_log, "MODULE %s already loaded\n", mod_id);
    log_msg(DEF_warning_log);
    dlclose(modulo[cont1]);
    goto Prox_modulo;
   }
  }
  // Registrando modulo
  selor_strncpy(mod_names[cont1].mod_id, mod_id, TAM_mod_name);
  mod_names[cont1].mod_index=cont1;
  snprintf(buffer_log, TAM_buffer_log, "(%s) loaded, id:%s, v:%s\n", DEF_module[cont1], mod_id, mod_version);
  log_msg(DEF_warning_log);
  // Vamos registra-lo para todas as consultas
  modseq_auth[cont1]=cont1;
  modseq_domain[cont1]=cont1;
  modseq_mail[cont1]=cont1;
  modseq_alias[cont1]=cont1;
  //modseq_quota[cont1]=cont1;
  //modseq_quota[cont1]=0xFF;
  if(ret1)*ptr_aux2=ret1;
  Prox_modulo:;
 }
 
 if(mod_refresh_flag){
  // -- Criando a Thread para dar refresh nas conexoes dos modulos --
  temp1=pthread_create(&thmod_refresh, NULL, mod_refresh, NULL);
 }
 
 modseq_auth[cont1]=0xFF;
 modseq_domain[cont1]=0xFF;
 modseq_mail[cont1]=0xFF;
 modseq_alias[cont1]=0xFF;
 // Reaplicando definicoes dos modseq...
 if(DEF_modseq_auth[0])seq_modules(modseq_auth, DEF_modseq_auth);
 if(DEF_modseq_domain[0])seq_modules(modseq_domain, DEF_modseq_domain);
 if(DEF_modseq_mail[0])seq_modules(modseq_mail, DEF_modseq_mail);
 if(DEF_modseq_alias[0])seq_modules(modseq_alias, DEF_modseq_alias);

 // == Modulo TLS
 if(DEF_tls_module[0]){
  ret1=0;
  if((tls_mod=dlopen(DEF_tls_module, RTLD_NOW | RTLD_LOCAL))){
   tls_load=dlsym(tls_mod, "tls_load");
   tls_init=dlsym(tls_mod, "tls_init");
   tls_connect=dlsym(tls_mod, "tls_connect");
   tls_accept=dlsym(tls_mod, "tls_accept");
   tls_close=dlsym(tls_mod, "tls_close");
   tls_send=dlsym(tls_mod, "tls_send");
   tls_recv=dlsym(tls_mod, "tls_recv");
   tls_error=dlsym(tls_mod, "tls_error");
   strcpy(buffer_log, "Invalid tls module\n");
  }else snprintf(buffer_log, TAM_buffer_log, "dlerror: %s\n", dlerror());
  if(!tls_load || !tls_init || !tls_connect || !tls_accept || 
     !tls_close || !tls_send || !tls_recv || !tls_error || 
     (ret1=tls_load(DEF_tls_module_p, DEF_tls_cafile, DEF_tls_privkey, DEF_tls_pubkey))){
   if(ret1)snprintf(buffer_log, TAM_buffer_log, "Tls_module: %s\n", tls_error());
   log_msg(DEF_warning_log);
   return 1;
  }
 }
  
 tam_mem_rcpts=sizeof(struct dados_email) * DEF_max_rcpts;
 to=malloc(tam_mem_rcpts);                   // Alocacao de memoria para a lista de destinatarios
 mx_map=malloc(sizeof(struct str_mx_map) * DEF_max_rcpts);
 mx_index=malloc(sizeof(int) * DEF_max_rcpts);
 if(!to || !mx_map || !mx_index){
  selor_superstrncpy(buffer_log, TAM_buffer_log, "ERROR: ", strerror(errno), "\n", NULL);
  log_msg(DEF_warning_log);
  close(arq_pid);
  return 1;
 }
 atual_rcpt=to;
 memset(to, 0, tam_mem_rcpts);
 
 if(param_p){
  setgid(gid);          // Alterando GID do processo 
  selor_setuid(uid);    // Alterando UID do processo 
  strcpy(fpid, itoa(getpid(), 0));
  signal(SIGCHLD, SIG_DFL);         // Para funcao 'system' ret. valores     
  if(param_qp)selor_strncat(DEF_queue_path, "/qlimit", TAM_defs);
  ret1=proc_id(param_p);
  if(to)free(to);
  for(cont1=0; cont1<num_mods; cont1++){
   if(modulo[cont1]){
    mod_shutdown[cont1]();
    dlclose(modulo[cont1]);
   }
  }
  return ret1;
 }
 
 // ---------------------------------
 //  Programa que trata a FILA       
 // ---------------------------------
 if(!param_q){
  qproc=fork();
  if(!qproc){
   // ------------------------------------------
   // -- Processo Filho que vai tratar a fila --
   setgid(gid);          // Alterando GID do processo 
   selor_setuid(uid);    // Alterando UID do processo 
   pid_slots[0].dir=opendir(DEF_queue_path);
   if(!pid_slots[0].dir){
    selor_superstrncpy(buffer_log, TAM_buffer_log, "ERROR: The path can't be opened: ", DEF_queue_path, NULL);
    log_msg(DEF_warning_log);
    if(to)free(to);
    return 1;
   }
   selor_superstrncpy(buffer_log, TAM_buffer_log, DEF_queue_path, "/qlimit", NULL);
   if((pid_slots[1].dir=opendir(buffer_log)))cont2=1;
   else cont2=0;
   pid_slots[0].qtime=DEF_queue_time;
   pid_slots[1].qtime=DEF_qlimit_time;
   if(DEF_max_qprocs){
    pid_slots[0].slots=malloc(DEF_max_qprocs*sizeof(int));
    pid_slots[0].n_slots=DEF_max_qprocs;
    memset(pid_slots[0].slots, 0, DEF_max_qprocs*sizeof(int));
   }
   if(DEF_qlimit_max_procs){
    pid_slots[1].slots=malloc(DEF_qlimit_max_procs*sizeof(int));
    pid_slots[1].n_slots=DEF_qlimit_max_procs;
    selor_strncpy(pid_slots[1].subdir, "qlimit", TAM_subdirs);
    memset(pid_slots[1].slots, 0, DEF_qlimit_max_procs*sizeof(int));
   }
   while(1){
    if(getloadavg(&load_avg, 1) && load_avg < (double)DEF_lavg_queue){
     time(&temp_queue1);
     for(cont3=0; cont3<=cont2; cont3++){
      if(!cont3)chdir(DEF_queue_path);
      else chdir(pid_slots[cont3].subdir);
      if(pid_slots[cont3].n_slots){
       pid_slots[cont3].livres=compute_slots(pid_slots[cont3].slots, pid_slots[cont3].n_slots);     // Computando slots livres e usados
       if(!pid_slots[cont3].livres)continue;     // Todos os slots usados
      }
      while((entr = readdir(pid_slots[cont3].dir))){
       stat(entr->d_name, &dados_entr);
       if(entr->d_name[0] == 'c' && !(dados_entr.st_mode & S_IFDIR)){
        ptr_aux1=strchr(entr->d_name, '_');
        if(ptr_aux1){
         *ptr_aux1=0;
         temp_queue2=atoi(&entr->d_name[1]);
         *ptr_aux1='_';
         cont1=(unsigned int)difftime(temp_queue1, temp_queue2);
         t_restante=(cont1/60)%pid_slots[cont3].qtime;
         if(t_restante != pid_slots[cont3].qtime-1)continue;
         // -- Verificando Qlimits --
         if(pid_slots[0].n_slots && !pid_slots[0].livres)break;      // Controle GLOBAL de todas as filas
         if(cont3 && pid_slots[cont3].n_slots && !pid_slots[cont3].livres)break;
         temp1=fork();
         if(!temp1){
          signal(SIGCHLD, SIG_DFL);         // Para funcao 'system' ret. valores     
          strcpy(fpid, itoa(getpid(), 0));
          if(cont3){
           selor_strncat(DEF_queue_path, "/", TAM_defs);
           selor_strncat(DEF_queue_path, pid_slots[cont3].subdir, TAM_defs);
          }
          ret1=proc_id(&entr->d_name[1]);
          for(cont1=0; cont1<num_mods; cont1++)if(modulo[cont1])dlclose(modulo[cont1]);
          if(to)free(to);
          exit(0);
         }
         if(pid_slots[0].n_slots){
          // Controle da queue GLOBAL
          reg_pid(pid_slots[0].slots, pid_slots[0].n_slots, temp1);
          pid_slots[0].livres--;
         }
         if(cont3 && pid_slots[cont3].n_slots){
          reg_pid(pid_slots[cont3].slots, pid_slots[cont3].n_slots, temp1);
          pid_slots[cont3].livres--;
         }
        }
       }
      }
      rewinddir(pid_slots[cont3].dir);
     }
    }
    sleep(DEF_qsleep_time);
   }
  }else if(qproc < 0){
   // -- Erro no FORK
   selor_superstrncpy(buffer_log, TAM_buffer_log, "ERROR: Fork error in QUEUE Process (", strerror(errno), ")\n", NULL);
   log_msg(DEF_warning_log);
   if(to)free(to);
   return 1;
  }
 }
 
 // Preparando buffer da LIMIT
 if(DEF_limit_buf){
  dosd_buffer=malloc(DEF_limit_buf * 8);
  memset(dosd_buffer, 0, DEF_limit_buf * 8);
  dosd_ctrl.mem=dosd_buffer;
  dosd_ctrl.fim_mem=dosd_buffer + (DEF_limit_buf * 8);
  dosd_ctrl.atual_posi=dosd_buffer;
  dosd_ctrl.intervalo=DEF_limit_time;
  dosd_ctrl.num_ocorr=DEF_limit_cnx;
 }
 
 signal(SIGTERM, sinal_sigterm);
 // ---------------------------------
 //  Criando SOCKET                  
 // ---------------------------------
 main_socket=socket(AF_INET, SOCK_STREAM, 0);
 rede_local.sin_family=AF_INET;
 rede_local.sin_port=htons(DEF_port);
 rede_local.sin_addr.s_addr=0;
 memset(&rede_local.sin_zero, 0, 8);
 if(main_socket == -1 || (setsockopt(main_socket, SOL_SOCKET, SO_REUSEADDR, &num_socket, sizeof(int)) == -1) || (bind(main_socket, (struct sockaddr *)&rede_local, sizeof(rede_local))) == -1){
  // Erro no SOCKET
  selor_superstrncpy(buffer_log, TAM_buffer_log, "ERROR: ", strerror(errno), "\n", NULL);
  log_msg(DEF_warning_log);
  sinal_sigterm(SIGTERM);    // Terminando execucao do servidor
 }
 // --  Escutando PORTA --
 listen(main_socket, 20);    // Maximo de 20 conexoes pendentes na fila
 selor_superstrncpy(buffer_log, TAM_buffer_log, "Listen port ", itoa(DEF_port, 0), "\n", NULL);
 log_msg(DEF_central_log);
 
 etapa_global=0.1;
 while(1){
  tam_socket=sizeof(struct sockaddr_in);
  socket_remoto=accept(main_socket, (struct sockaddr *)&rede_remota, &tam_socket);
  if(DEF_limit_buf){
   etapa_global=0.2;
   // Rodando prevencao contra DoS
   time(&tempo);
   dosd_ctrl.ip=rede_remota.sin_addr.s_addr;
   dosd_ctrl.timestamp=tempo;
   if(dosd(&dosd_ctrl)){
    etapa_global=0.2;
    // -- Limite atindigo --
    if(!fork()){
     if(DEF_event_limit[0]){
      // Pegando dados do host remoto
      host.ipn=rede_remota.sin_addr.s_addr;
      strcpy(host.ip, inet_ntoa(rede_remota.sin_addr));
      host.strlen_ip=strlen(host.ip);
      if(DEF_resolve_all){
       reverso(host.ipn, host.nome, NS_MAXDNAME-1);
       host.strlen_nome=strlen(host.nome);
      }
      etapa_global=0.3;
      exec_regras(DEF_event_limit, NULL, &from, atual_rcpt);
      etapa_global=0.4;
      if(R_ACCEPT)goto Cnx_liberada;
     }
     close(socket_remoto);
     if(to)free(to);
     return 0;
    }
    goto Fechar_socket_remoto;
   }
   if(!(temp1=fork()))goto Cnx_liberada_dados;         // Para nao rodar time novamente
   goto Fechar_socket_remoto;
  }
  
  temp1=fork();
  if(!temp1){
   // ---------------------------------------------
   // -- Processo Filho que vai tratar a conexao --
   etapa_global=0.5;
   time(&tempo);
   // Pegando dados do host remoto
   Cnx_liberada_dados:
   host.ipn=rede_remota.sin_addr.s_addr;
   strcpy(host.ip, inet_ntoa(rede_remota.sin_addr));
   host.strlen_ip=strlen(host.ip);
   if(DEF_resolve_all){
    reverso(host.ipn, host.nome, NS_MAXDNAME-1);
    host.strlen_nome=strlen(host.nome);
   }
   Cnx_liberada:
   close(main_socket);
   
   if(!DEF_timezone[0]){
    timeinfo=localtime(&tempo);
    sprintf(DEF_timezone, "%+.2d00", (int)(timeinfo->tm_gmtoff/3600)+timeinfo->tm_isdst);
   }
   
   strcpy(fpid, itoa(getpid(), 0));
   signal(SIGCHLD, SIG_DFL);         // Para funcao 'system' ret. valores     
   signal(SIGTERM, capta_sinal);     // Capturando sinais SIGTERM    
   setgid(gid);                      // Alterando GID do processo    
   selor_setuid(uid);                // Alterando UID do processo    
   
   // Lendo LOAD AVERAGE
   if(getloadavg(&load_avg, 1) && load_avg >= (double)DEF_lavg_negative){
    num_smtp_resp=MSG_lavg;
    smtp_resp(NULL, DEF_log_lvl, NVL_log_desativado);
    close(socket_remoto);
    if(to)free(to);
    return 0;
   }
   
   host.autenticado=0;
   selor_superstrncpy(buffer_log, TAM_buffer_log, "(PID ", fpid, ") ", host.ip, "(", host.nome, ") - Connected\n", NULL);
   log_msg(DEF_central_log);
   num_smtp_resp=MSG_welcome;
   R_DENY=0;
   etapa_global=0.6;
   if(DEF_event_connection[0])exec_regras(DEF_event_connection, NULL, &from, atual_rcpt);
   etapa_global=0.7;
   smtp_resp(NULL, DEF_log_lvl, NVL_log_env);
   if(R_DENY){
    close(socket_remoto);
    if(to)free(to);
    return 0;
   }
   // -- Gerando um ID de fila para a conexao --
   tam_id_fila=selor_superstrncpy(id_pid, TAM_id_fila, itoa(tempo, 0), "_", fpid, NULL);
   do{
    // Lendo comandos SMTP
    Novo_cmd:
    ptr_smtp_cmd=smtp_cmd;
    if((temp1=selor_cmd_recv(socket_remoto, smtp_cmd, TAM_smtp_cmd, 0)) < 1){
     // -- Conexao Interrompida --
     cnx_int=1;
     break;
    }
    ptr_smtp_cmd+=temp1-2;       // '-2' para retirar o /r/n do comando
    *ptr_smtp_cmd=0;

    // ****************************
    // *****     RCPT TO      *****
    // ****************************
    if(!cmp_str(smtp_cmd, "rcpt to:", 8) && etapa_smtp >= ETAPA_rcpt_to){
     etapa_global=0.8;
     if(num_rcpt >= DEF_max_rcpts){
      num_smtp_resp=MSG_toomany_rcpts;
      smtp_resp(NULL, DEF_log_lvl, NVL_log_rec_env);
      continue;
     }
     atual_rcpt=&to[num_rcpt];
     atual_rcpt->auth[0]=0;
     if(ptr_smtp_cmd-&smtp_cmd[8] > TAM_endereco || smtp_dados_email(&smtp_cmd[8], atual_rcpt, NULL) < 0 || !atual_rcpt->email[0] || atual_rcpt->email[0] == '@' || !atual_rcpt->dominio){
      num_smtp_resp=MSG_invalid_param;
      Invalidar_rcpt:
      if(DEF_event_lrcpt_refused[0]){
       selor_strncpy(R_SERVER_RESP, smtp_resps[num_smtp_resp], sizeof(R_SERVER_RESP));  // Copiando a reposta do servidor para a variavel '%T'
       exec_regras(DEF_event_lrcpt_refused, NULL, &from, atual_rcpt);
      }
      smtp_resp(atual_rcpt->email, DEF_log_lvl, NVL_log_rec_env);
      atual_rcpt->email[0]=0;
      continue;
     }
     if(proc_rcpt(atual_rcpt->email, 1))goto Novo_cmd;                  // Se ja estiver na lista, responde OK e pula para o proximo comando
     ret1=cache_mod_dom_local(atual_rcpt->dominio, adomain, &cont1);
     if(adomain[0] && proc_rcpt(atual_rcpt->email, 1))goto Novo_cmd;    // Se ja estiver na lista, responde OK e pula para o proximo comando
     atual_rcpt->status=STATUS_PENDENTE;
     prox_num_rcpt=num_rcpt+1;
     
     if(ret1){
      // -- Dominio interno ou erro --
      etapa_global=0.9;
      if(ret1 == 1){
       atual_rcpt->local=1;         // Dominio local
       R_ACCEPT=1;
       num_smtp_resp=MSG_ok;
       mod_user_info[MOD_user_user]=mod_user_info[MOD_user_pass]=mod_user_info[MOD_user_home]=mod_user_info[MOD_user_dlocal]=NULL;
       // Checando se e um SRS
       if(DEF_srs_key[0] && !cmp_str(atual_rcpt->email, "SRS", 3) 
                         && (atual_rcpt->email[3] == '0' || atual_rcpt->email[3] == '1')
                         && atual_rcpt->email[4] == '+' && !srs_decode(srs_rcpt, TAM_endereco, atual_rcpt->email, DEF_srs_key, DEF_srs_shift, tempo))goto RCPT_proc_regras;  // Para ir dereto a parte de checagem de regras
       
       //cont1=mod_user_local[0](atual_rcpt->email, mod_user_info, MOD_user_ncampos);
       cont1=0;
       while(!(ret_val=mod_user_local[modseq_mail[cont1]](atual_rcpt->email, mod_user_info, MOD_user_ncampos)) && (modseq_mail[++cont1] != 0xFF));
       cont2=DEF_force_alias_seek;
       cont3=0;
       if(ret_val == 1 || ret_val == 2){
        // -- Usuario encontrado --
        // Checando a quota
        if((mod_test_quota[modseq_mail[cont1]](atual_rcpt->email, mod_user_info, MOD_user_ncampos)) == 2){
         // Quota excedida
         num_smtp_resp=MSG_full_mailbox;
         goto Invalidar_rcpt;
        }
        if(mod_user_info[MOD_user_home])selor_strncpy(rcpt_info.home, mod_user_info[MOD_user_home], TAM_home);
        if(mod_user_info[MOD_user_dlocal])selor_strncpy(atual_rcpt->dest, mod_user_info[MOD_user_dlocal], TAM_email_dest);
        //atual_rcpt->local=1;         // Dominio local
        atual_rcpt->status=STATUS_PENDENTE;
        cont3=1;
       }
       if(!ret_val || ret_val == 2 || cont2){
        // Usuario nao existe. Vamos ver se existe ALIAS
        if(DEF_srs_key[0] && DEF_srs_alias_redir && !srs_from[0] && !from.local)ret1=1;
        else ret1=0;
        df_alias=0;
        rcpt_info.home[0]=0;
        memcpy(&tmp_email, atual_rcpt, sizeof(tmp_email));        // Salvando o rcpt para ser     
        atual_rcpt=&tmp_email;                                    //  usado nos arquivos de regras
        ptr_aux1=tmp_email.email;
        Buscar_pegatudo:
        cont1=0;
        prox_bloco=0;
        while(!(temp1=mod_ret_alias[modseq_alias[cont1]](ptr_aux1, ret_alias[0],  MAX_buffer_alias, TAM_endereco, &prox_bloco)) && (modseq_alias[++cont1] != 0xFF));
        if(temp1 < 0){
         // Erro no modulo
         tratar_erro_modulo(modseq_alias[cont1]);
         atual_rcpt->email[0]=0;     // Inutilizando o rcpt recem gravado
         continue;
        }
        if(!temp1){
         // Nao existe mesmo
         if(cont3)goto RCPT_proc_regras;             // Se um email normal foi adicionado.
         if(DEF_seek_df_alias && !df_alias){         // df = Default Alias @xxxxxxxxx. DEF_seek_df_alias = Seek Default Alias
           // Procurando conta do tipo pega tudo --> @dominio.com
           df_alias=1;
           if((ptr_aux1=strchr(ptr_aux1, '@')))goto Buscar_pegatudo;
          }
         num_smtp_resp=MSG_no_mailbox;
         if(DEF_no_mailbox_delay)sleep(DEF_no_mailbox_delay);
         goto Invalidar_rcpt;
        }
        // Alias encontrado
        if(!cont3)prox_num_rcpt--;
        while(1){                                    // Copiando emails para a lista de destinatarios
         if(prox_num_rcpt+temp1 > DEF_max_rcpts){
          num_smtp_resp=MSG_toomany_rcpts;           // Numero de alias estoura o limite de destinatarios da conexao
          goto Invalidar_rcpt;                       // Destinatario ser ignorado                                  
         }
         for(cont2=0; cont2<temp1; cont2++){
          if(ret_alias[cont2][0] == '@' && DEF_allow_broadcasts){
           // -- Broadcast --
           ptr_aux2=&ret_alias[cont2][1];
           if(*ptr_aux2){
            // Fazendo chamada a funcao broadcast do modulo para pegar todos os emails do dominio
            cont4=0;
            prox_bloco_broadcast=0;
            while((!mod_ret_broadcast[modseq_mail[cont4]] || !(temp3=mod_ret_broadcast[modseq_mail[cont4]](ptr_aux2, ret_broadcast[0],  MAX_buffer_broadcast, TAM_endereco, &prox_bloco_broadcast))) && (modseq_mail[++cont4] != 0xFF));
            if(temp3 > 0){
             while(1){
              if(prox_num_rcpt+temp3 > DEF_max_rcpts){
               num_smtp_resp=MSG_toomany_rcpts;           // Numero de alias estoura o limite de destinatarios da conexao
               goto Invalidar_rcpt;                       // Destinatario ser ignorado                                  
              }
              for(cont5=0; cont5<temp3; cont5++){
               for(temp4=0; temp4<prox_num_rcpt; temp4++)if(!strcmp(to[temp4].email, ret_broadcast[cont5]))break;    // Checando se ja nao esta cadastrado
               if(temp4 == prox_num_rcpt){
                smtp_dados_email(ret_broadcast[cont5], &to[prox_num_rcpt], NULL);    // Incluindo lista de alias
                to[prox_num_rcpt].local=1;        // Dominio Interno
                to[prox_num_rcpt++].status=STATUS_PENDENTE;
               }
              }
              if(temp3 < MAX_buffer_broadcast)break;
              temp3=mod_ret_broadcast[modseq_mail[cont4]](ptr_aux2, ret_broadcast[0],  MAX_buffer_broadcast, TAM_endereco, &prox_bloco_broadcast);
             }
            }else if(temp3 == -1){
             tratar_erro_modulo(modseq_mail[cont4]);
             atual_rcpt->email[0]=0;       // Inutilizando o rcpt recem gravado
             continue;
            }
           }
          }else {
           // -- Alias Normal --
           for(temp2=0; temp2<prox_num_rcpt; temp2++)if(!strcmp(to[temp2].email, ret_alias[cont2]))break;        // Checando se ja nao esta cadastrado
           if(temp2 == prox_num_rcpt){
            smtp_dados_email(ret_alias[cont2], &to[prox_num_rcpt], NULL);    // Incluindo lista de alias
            if(ret1 && !srs_from[0]){
             // SRS ligado. Temos que ver se o dominio  externo para aplicar o SRS
             if(!cache_mod_dom_local(to[prox_num_rcpt].dominio, adomain, &t_restante)){
              if(!srs_encode(srs_from, TAM_endereco, from.email, DEF_server_name, DEF_srs_key, DEF_srs_shift, tempo))smtp_dados_email(srs_from, &from, NULL);
              to[prox_num_rcpt].local=0;           // Dominio externo
             }else to[prox_num_rcpt].local=1;      // Dominio interno
            }else to[prox_num_rcpt].local=-1;      // -1 = Indefinido
            to[prox_num_rcpt++].status=STATUS_PENDENTE;
           }
          }
         }
         if(temp1 < MAX_buffer_alias)break;
         temp1=mod_ret_alias[modseq_alias[cont1]](ptr_aux1, ret_alias[0],  MAX_buffer_alias, TAM_endereco, &prox_bloco);
        }
       }else if(!cont3){
        // Erro no modulo
        tratar_erro_modulo(modseq_alias[cont1]);
        atual_rcpt->email[0]=0;       // Inutilizando o rcpt recem gravado
        continue;
       }
      }else {
       // Erro no modulo
       tratar_erro_modulo(modseq_domain[cont1]);
       atual_rcpt->email[0]=0;        // Inutilizando o rcpt recem gravado
       continue;
      }
     }else {
      // -- Dominio Externo --
      if(DEF_rcpt_level_check && dns_query(atual_rcpt->dominio, T_MX, &cont1) && !resolver_dom(atual_rcpt->dominio, NULL, 0)){
       num_smtp_resp=MSG_invalid_domain;
       if(DEF_event_lrcpt_refused[0])exec_regras(DEF_event_lrcpt_refused, NULL, &from, atual_rcpt);
       smtp_resp(atual_rcpt->dominio, DEF_log_lvl, NVL_log_rec_env);
       atual_rcpt->email[0]=0;
       continue;
      }
      atual_rcpt->local=0;    // Dominio exerno
      atual_rcpt->status=STATUS_PENDENTE;
      R_ACCEPT=R_RELAY | (host.autenticado & (char)DEF_auth_relay);
      if(R_ACCEPT)num_smtp_resp=MSG_ok;
      else num_smtp_resp=MSG_relay_denied;
     }
     RCPT_proc_regras:
     R_DENY=0;
     atual_rcpt->so_mark=R_SO_MARK;
     etapa_global=1.0;
     if(DEF_event_rcpt[0])exec_regras(DEF_event_rcpt, NULL, &from, atual_rcpt);
     etapa_global=1.1;
     smtp_resp(NULL, DEF_log_lvl, NVL_log_rec_env);
     if(R_ACCEPT && !R_DENY){
      if(srs_rcpt[0]){
       // SRS valido. Vamos trocar o endereco --
       smtp_dados_email(srs_rcpt, atual_rcpt, NULL);
       srs_rcpt[0]=0;
      }
      num_rcpt=prox_num_rcpt;         // num_rcpt = proximo indice do rcpt
      etapa_smtp=ETAPA_data;
     }
    
    // ****************************
    // *****    MAIL FROM     *****
    // ****************************
    }else if(!cmp_str(smtp_cmd, "mail from:", 10) && etapa_smtp >= ETAPA_mail_from){
     rset();
     etapa_global=1.2;
     if(ptr_smtp_cmd-&smtp_cmd[8] > TAM_endereco || smtp_dados_email(&smtp_cmd[10], &from, &tam_email) < 0 || from.email[0] == '@'){
      num_smtp_resp=MSG_invalid_param;
      smtp_resp(from.email, DEF_log_lvl, NVL_log_rec_env);
      from.email[0]=0;
      continue;
     }
     if(DEF_max_mail_size && tam_email > DEF_max_mail_size){      // Checando se o tamanho do email e maior que permitido
      num_smtp_resp=MSG_mail_toobig;
      smtp_resp(NULL, DEF_log_lvl, NVL_log_rec_env);
      etapa_smtp=ETAPA_mail_from;
      continue;
     }
     num_smtp_resp=MSG_ok;
     R_ACCEPT=1;
     if(from.dominio){
      ret1=cache_mod_dom_local(from.dominio, adomain, &cont1);
      if(ret1){
       // -- Dominio interno ou erro --
       if(ret1 == 1){
        from.local=1;      // Dominio local
        mod_user_info[MOD_user_user]=mod_user_info[MOD_user_pass]=mod_user_info[MOD_user_home]=mod_user_info[MOD_user_dlocal]=NULL;
        //if(!mod_user_local[0](from.email, mod_user_info, MOD_user_ncampos)){
        cont1=0;
        while(!(ret_val=mod_user_local[modseq_mail[cont1]](from.email, mod_user_info, MOD_user_ncampos)) && (modseq_mail[++cont1] != 0xFF));
        if(!ret_val){
         // Email de remetente nao existe
         from_info.home[0]=0;
         cont1=0;
         prox_bloco=0;
         while(!(temp1=mod_ret_alias[modseq_alias[cont1]](from.email, ret_alias[0], 1, TAM_endereco, &prox_bloco)) && (modseq_alias[++cont1] != 0xFF));
         if(!temp1){
          if(DEF_no_mailbox_delay)sleep(DEF_no_mailbox_delay);
          num_smtp_resp=MSG_no_mailbox;
          smtp_resp(from.email, DEF_log_lvl, NVL_log_rec_env);
          continue;
         }else if(temp1 < 0){
          tratar_erro_modulo(modseq_alias[cont1]);
          continue;
         }
         // Existe e e um ALIAS
         from_info.home[0]=0;
         from.local=-1;      // -1 = Indefinido. Aparentemente e' local, mas sera testado depois
        }else {
         if(ret_val == -1){
          // Erro no modulo
          tratar_erro_modulo(modseq_domain[cont1]);
          continue;
         }
         if(mod_user_info[MOD_user_home])selor_strncpy(from_info.home, mod_user_info[MOD_user_home], TAM_home);
         if(mod_user_info[MOD_user_dlocal])selor_strncpy(from.dest, mod_user_info[MOD_user_dlocal], TAM_email_dest);
         //from.local=1;      // Dominio local
        }
       }else {
        // Erro no modulo
        tratar_erro_modulo(modseq_domain[cont1]);
        continue;
       }
      }else {
       if(DEF_from_level_check && from.email[0]){
        if(from.dominio){
         if(DEF_from_level_check == 1 && resolver_dom(from.dominio, NULL, 0))goto FROM_dom_ok;
         if(!dns_query(from.dominio, T_MX, &cont1))goto FROM_dom_ok;
        }
        num_smtp_resp=MSG_invalid_domain;
        smtp_resp(from.dominio, DEF_log_lvl, NVL_log_rec_env);
        from.email[0]=0;
        continue;
       }
       FROM_dom_ok:
       from.local=0;  // -- Dominio externo --
      }
     }
     R_DENY=0;
     etapa_global=1.3;
     if(DEF_event_from[0])exec_regras(DEF_event_from, NULL, &from, atual_rcpt);
     etapa_global=1.4;
     smtp_resp(NULL, DEF_log_lvl, NVL_log_rec_env);
     if(R_ACCEPT && !R_DENY)etapa_smtp=ETAPA_rcpt_to;
     
     // ****************************
     // *****       DATA       *****
     // ****************************
    }else if(!cmp_str(smtp_cmd, "data\0", 5) && etapa_smtp >= ETAPA_data){
     etapa_global=1.5;
     if(R_DATA_REPLY[0]){
      selor_send(socket_remoto, R_DATA_REPLY, strlen(R_DATA_REPLY), 0);
      continue;
     }
     selor_superstrncpy(id_fila, TAM_id_fila, id_pid, itoa(msg_cont, 0), NULL);
     selor_superstrncpy(cam_filac, sizeof(cam_filac), DEF_queue_path, "/", R_DSPOOL, "/c", id_fila, NULL);
     selor_superstrncpy(cam_filal, sizeof(cam_filal), DEF_queue_path, "/", R_DSPOOL, "/l", id_fila, NULL);
     arq_lid=fopen(cam_filal, "wb");
     arq_lock=open(cam_filal, O_RDWR);
     if(arq_lock < 0)goto Erro_criando_arq;             // Esta parte fica primeiro das fopen abaixo pois queremos criar      
     if(lockf(arq_lock, F_TLOCK, 0) !=0){               // o arquivo lxxxx antes do cxxxxx, pois o processo de                
      close(arq_lock);                                  // fila busca por arquivo cxxxx e NAO queremos uma condio de corrida
      falha_interna(1);
     }
     arq_cid=fopen(cam_filac, "wb");
     if(!arq_cid || !arq_lid){
      // Erro ao criar o arquivo de email
      Erro_criando_arq:
      lockf(arq_lock, F_ULOCK, 0);
      close(arq_lock);
      selor_superstrncpy(buffer_log, TAM_buffer_log, "ERROR: (PID ", fpid, ") ", host.ip, " - Error creating queue file [", cam_filac, "] or [", cam_filal,"]\n", NULL);
      falha_interna(1);   // Avisa o host de Falha interna, faz LOG e finaliza
     }
     rem_arq_signal=1;
     num_smtp_resp=MSG_send_body;
     smtp_resp(NULL, DEF_log_lvl, NVL_log_rec_env);
     // Gravando destinatarios no arquivo de fila
     sem_espaco=gravar_lista(num_rcpt, arq_lid, cam_filal, 0, 0);       // --> Colocando depois de enviar MSG_send_body para so gravar local enquanto o cliente remoto se prepara para enviar
     // Gravando Received
     rfc_date(buffer, tempo);
     cont1=selor_superstrncpy(linha, TAM_buffer, "Return-Path: <", from.email, ">\nReceived: from ", host.helo, " (",
                              host.nome, "[", host.ip,"])\n", header_info_ini, header_info_auth, header_info_tls, header_info_fim,
                              "\tby ", DEF_server_name, " SMTP SELOR ", VERSAO, " id ", id_fila, "\n\t", NULL);
     if(num_rcpt == 1)cont1+=selor_superstrncpy(&linha[cont1], TAM_buffer-cont1, "for <", to[0].email, ">; ", buffer, "\n", NULL);
     else cont1+=selor_superstrncpy(&linha[cont1], TAM_buffer-cont1, buffer, "\n", NULL);
     cont1+=selor_strncpy(&linha[cont1], R_HADD, TAM_buffer-cont1);
     fwrite(linha, 1, cont1, arq_cid);
     memset(buffer, 0, 4);
     ptr_aux1=&buffer[4];
     falta_receber=TAM_buffer;
     tam_email=0;
     cont1=2;
     ptr_aux3=NULL;
     linha[0]=linha[1]=0;
     etapa_global=1.6;
     
     while(1){
      // ----------------------------------
      // -- LOOP do Recebimento de email --
      if((b_recebidos=selor_recv(socket_remoto, ptr_aux1, falta_receber, 0)) < 1){
       // Conexao perdida
       if(b_recebidos < 0)selor_superstrncpy(buffer_log, TAM_buffer_log, "(PID ", fpid, ") ", host.ip," - Connection timeout\n", NULL);
       else selor_superstrncpy(buffer_log, TAM_buffer_log, "(PID ", fpid, ") ", host.ip, " - Conn. broken in 'DATA'\n", NULL);
       log_msg(DEF_central_log);
       close(socket_remoto);
       fclose(arq_cid);
       fclose(arq_lid);
       remove(cam_filac);
       remove(cam_filal);
       lockf(arq_lock, F_ULOCK, 0);
       close(arq_lock);
       goto Proc_ids;
      }
      falta_receber-=b_recebidos;
      ptr_aux1+=b_recebidos;
      testa_fim_data=(unsigned int *)&ptr_aux1[-4];
      if(*testa_fim_data == 0xA0D2E0A)break;         // 0xA0D2E0A = '\n.\r\n'
      if(!falta_receber){
       // Buffer cheio. Vamos descarrega-lo no arquivo;
       cont1-=2;
       cont1=conv_crlf(linha, &buffer[4], TAM_buffer, &linha[cont1], &ptr_aux3);
       if(!tam_header && ptr_aux3)tam_header=tam_email+(ptr_aux3-linha);
       tam_email+=cont1;
       if(tam_email <= DEF_max_mail_size || !DEF_max_mail_size){
        temp1=fwrite(linha, 1, cont1, arq_cid);    // Gravando no arquivo se o tamanho maximo ainda nao foi excedido
        if(temp1 != cont1)sem_espaco=1;
       }
       memcpy(buffer, &buffer[TAM_buffer], 4);
       falta_receber=TAM_buffer;
       ptr_aux1=&buffer[4];
      }
     }
     // Buffer restante a gravar
     etapa_global=1.7;
     cont1-=2;
     cont2=TAM_buffer-falta_receber;
     ret1=0;
     if(cont2 > 3){
      if(tam_email <= DEF_max_mail_size || !DEF_max_mail_size){
       cont1=conv_crlf(linha, &buffer[4], TAM_buffer-falta_receber, &linha[cont1], &ptr_aux3);
       cont1-=2;
       temp1=fwrite(linha, 1, cont1, arq_cid);    // Gravando no arquivo se o tamanho maximo ainda nao foi excedido
       if(temp1 != cont1)sem_espaco=1;
      }
     }else {
      if(cont2 < 3)ret1=1;
      cont1=0;
     }
     fflush(arq_cid);
     fclose(arq_cid);
     tam_email+=cont1;
     if(ret1)truncate(cam_filac, --tam_email);
     if(!tam_header){
      if(ptr_aux3)tam_header=tam_email+(ptr_aux3-linha);
      else tam_header=tam_email;
     }
     etapa_smtp=ETAPA_mail_from;          // Resetando a etapa do protocolo
     if((DEF_max_mail_size && tam_email >= DEF_max_mail_size) || (tam_header >= DEF_max_header_size) || sem_espaco ){
      // Email muito grande ou Header muito grande
      if(tam_header >= DEF_max_header_size)num_smtp_resp=MSG_header_toobig;
      else if(sem_espaco)num_smtp_resp=MSG_nodisk_space;
      else num_smtp_resp=MSG_mail_toobig;
      smtp_resp(NULL, DEF_log_lvl, NVL_log_env);
      rem_arq_signal=0;
      fclose(arq_lid);
      remove(cam_filac);
      remove(cam_filal);
      lockf(arq_lock, F_ULOCK, 0);
      close(arq_lock);
      continue;    // Pula para a escuta de um novo comando SMTP
     }
     // Mensagem aceita
     selor_superstrncpy(buffer, TAM_buffer, "(ID: ", id_fila, ") ", itoa(tam_email>>10, 0), " Kb", NULL);
     R_DENY=0;
     R_ACCEPT=1;
     num_smtp_resp=MSG_mail_accepted;
     etapa_global=1.8;
     if(DEF_event_data[0])exec_regras(DEF_event_data, NULL, &from, atual_rcpt);
     etapa_global=1.9;
     rem_arq_signal=0;
     if(R_ACCEPT && !R_DENY){
      smtp_resp(buffer, DEF_log_lvl, NVL_log_env);
      msg_cont++;
      if(msg_cont >= DEF_max_msgs_per_cnx)close(socket_remoto);     // Permitir no maximo DEF_max_msgs_per_cnx mensagens por conexao
      num_rcpts=num_rcpt;
     }else {
      remove(cam_filac);
      remove(cam_filal);
      smtp_resp(NULL, DEF_log_lvl, NVL_log_env);
      num_rcpts=0;
     }
     fclose(arq_lid);
     lockf(arq_lock, F_ULOCK, 0);
     close(arq_lock);
      
     
    // ****************************
    // *****    EHLO/HELO     *****
    // ****************************
    }else if(!cmp_str(smtp_cmd, "ehlo ", 5) || !cmp_str(smtp_cmd, "helo ", 5)){
     etapa_global=2.0;
     if(ptr_smtp_cmd-&smtp_cmd[5] > TAM_helo){
      num_smtp_resp=MSG_invalid_param;
      smtp_resp(NULL, DEF_log_lvl, NVL_log_rec_env);
     }else{
      ptr_smtp_cmd=&smtp_cmd[5];
      ptr_smtp_cmd=pular_carac(ptr_smtp_cmd, ' ');
      if((ptr_aux1=strchr(ptr_smtp_cmd, ' ')))*ptr_aux1=0;
      selor_strncpy(host.helo, ptr_smtp_cmd, TAM_helo);
      if(!t_inject_helo(ptr_smtp_cmd)){         // Testando se existe caracteres invalidos
	      host.strlen_helo=ptr_aux1-ptr_smtp_cmd;
       R_DENY=0;
       R_ACCEPT=1;
       if(DEF_event_helo[0])exec_regras(DEF_event_helo, NULL, &from, atual_rcpt);
	      if(R_ACCEPT && !R_DENY){
        etapa_smtp=ETAPA_mail_from;
        if(smtp_cmd[0]=='h' || smtp_cmd[0]=='H'){
         // Resposta ao comando HELO 
         temp1=selor_superstrncpy(buffer, TAM_buffer, "250 ", DEF_server_name, "\r\n", NULL);
         selor_send(socket_remoto, buffer, temp1, 0);
         if(!tls_ativo){
          selor_superstrncpy(buffer_log, TAM_buffer_log, "(PID ", fpid, ") <- ", host.ip, " ", smtp_cmd, "\n", NULL);
          log_msg(DEF_central_log);
         }
        }else {
		       num_smtp_resp=MSG_resp_ehlo;
         smtp_resp(NULL, DEF_log_lvl, NVL_log_rec-tls_ativo);   // Resposta ao comando EHLO. Se tls estiver ativo, ele nao fara log do EHLO
        }
       }else smtp_resp(NULL, DEF_log_lvl, NVL_log_env);
      }else {
       num_smtp_resp=MSG_invalid_param;
       smtp_resp(NULL, DEF_log_lvl, NVL_log_rec_env);
      }
     }

     
    // ****************************
    // ***** AUTH PLAIN/LOGIN *****
    // ****************************
    //}else if(!cmp_str(smtp_cmd, "auth ", 5) && etapa_smtp >= ETAPA_mail_from){    // Esta linha nao permite autenticacao antes de HELO/EHLO e foi desativada por enquanto
    }else if(!cmp_str(smtp_cmd, "auth ", 5)){
     etapa_global=2.1;
     ptr_aux2=buffer;
     if(!cmp_str(&smtp_cmd[5], "plain ", 6)){
     // === AUTH PLAIN ===
      ptr_aux1=&smtp_cmd[11];
      smtp_cmd[10]=0;
      cont1=base64_decod(ptr_aux2, ptr_aux1);
      ptr_aux2[cont1]=0;          //
      ptr_aux2[cont1+1]=0;        //  Protecao contra buffers invalidos 
      ptr_aux2[cont1+2]=0;        //
      ptr_aux2=strchr(ptr_aux2, 0);
      while(*ptr_aux2 != 0)ptr_aux2++;
      ptr_aux2++;
      strcpy(auth_user, ptr_aux2);
      while(*ptr_aux2 != 0)ptr_aux2++;
      ptr_aux2++;
      strcpy(auth_pass, ptr_aux2);
     }else if(!cmp_str(&smtp_cmd[5], "login", 5)){
      // === AUTH LOGIN ===
      ptr_aux1=&smtp_cmd[10];
      if(!*ptr_aux1){
       num_smtp_resp=MSG_auth_login_u;
       smtp_resp(NULL, DEF_log_lvl, NVL_log_desativado);
       if((temp1=selor_cmd_recv(socket_remoto, linha, TAM_buffer, 0)) < 1)goto Cnx_interrompida;
       ptr_aux1=linha;
      }else if(*ptr_aux1 != ' '){
       num_smtp_resp=MSG_invalid_cmd;
       smtp_resp(NULL, DEF_log_lvl, NVL_log_rec_env);
       continue;
      }else ptr_aux1++;
      smtp_cmd[10]=0;
      ptr_aux1=pular_carac(ptr_aux1, ' ');
      base64_decod(ptr_aux2, ptr_aux1);
      strcpy(auth_user, ptr_aux2);
      num_smtp_resp=MSG_auth_login_p;
      smtp_resp(NULL, DEF_log_lvl, NVL_log_desativado);
      if((temp1=selor_cmd_recv(socket_remoto, linha, TAM_buffer, 0)) < 1)goto Cnx_interrompida;
      base64_decod(ptr_aux2, linha);
      strcpy(auth_pass, ptr_aux2);
     }else {
      num_smtp_resp=MSG_invalid_cmd;
      smtp_resp(NULL, DEF_log_lvl, NVL_log_rec_env);
      continue;
     }
     mod_user_info[MOD_user_user]=mod_user_info[MOD_user_pass]=mod_user_info[MOD_user_home]=mod_user_info[MOD_user_dlocal]=NULL;
     selor_strncpy(auth_info.user, auth_user, TAM_endereco);
     cont1=0;
     while(!(ret_val=mod_auth[modseq_auth[cont1]](auth_user, auth_pass, mod_user_info, MOD_user_ncampos)) && (modseq_auth[++cont1] != 0xFF));
     if(ret_val == 1){
      if(mod_user_info[MOD_user_home])selor_strncpy(auth_info.home, mod_user_info[MOD_user_home], TAM_home);
      num_smtp_resp=MSG_auth_ok;
      selor_superstrncpy(buffer_log, TAM_buffer_log, "(PID ", fpid, ") <- ", host.ip, " ", smtp_cmd, " (", auth_user, ": OK)\n", NULL);
      host.autenticado=1;
      if(!DEF_hshow_user)strcpy(header_info_auth,"authenticated");
      else selor_superstrncpy(header_info_auth, sizeof(header_info_auth), "authenticated=", auth_user, NULL);
      header_info_ini[0]='\t';
      header_info_fim[0]=')';
     }else {
      auth_tolerance_cont1++;
      if(auth_tolerance_cont1 >= DEF_auth_tolerance){
       if(to)free(to);
       return 1;
      }
      sleep(DEF_incorrect_pass_delay);
      if(ret_val == -1){
       tratar_erro_modulo(cont1);          // -> Erro no modulo 
       continue;
      }
      num_smtp_resp=MSG_auth_falied;
      selor_superstrncpy(buffer_log, TAM_buffer_log, "(PID ", fpid, ") <- ", host.ip, " ", smtp_cmd, " (", auth_user, ": FALIED)\n", NULL);
     }
     smtp_resp(NULL, DEF_log_lvl, NVL_log_desativado);
     log_msg(DEF_central_log);
     
    // ****************************
    // *****       QUIT       *****
    // ****************************
    }else if(!cmp_str(smtp_cmd, "quit\0", 5)){
     num_smtp_resp=MSG_goodbye;
     smtp_resp(NULL, DEF_log_lvl, NVL_log_rec);
     manter_conectado=0;
     
    // ****************************
    // *****       RSET       *****
    // ****************************
    }else if(!cmp_str(smtp_cmd, "rset\0", 5)){
     if(DEF_immed_delivery == 2 && msg_cont){
      // Criando outro processo para entregar o email
      temp1=fork();
      if(!temp1)goto Immed2;
      else if(temp1 < 0){
       selor_superstrncpy(buffer_log, TAM_buffer_log, "ERROR: PID (", fpid, ") Fork fail\n", NULL);
       log_msg(DEF_warning_log);
       if(to)free(to);
       return 1;
      }
     }
     num_smtp_resp=MSG_ok;
     smtp_resp(NULL, DEF_log_lvl, NVL_log_rec);
     rset();
    
    // ****************************
    // *****    STARTTLS      *****
    // ****************************
    }else if(DEF_tls_enable && !tls_ativo && !cmp_str(smtp_cmd, "starttls\0", 9)){
     etapa_global=2.2;
     if(!tls_init(DEF_tls_module_p, socket_remoto, DEF_tls_cafile, DEF_tls_privkey, DEF_tls_pubkey)){
      selor_send(socket_remoto, "220 TLS Ready\r\n", 15, 0);
      selor_superstrncpy(buffer_log, TAM_buffer_log, "(PID ", fpid, ") <- ", host.ip, " ", smtp_cmd, "\n", NULL);
      log_msg(DEF_central_log);
      temp=0;
      alarm(DEF_timeout);
      temp=tls_accept();
      alarm(0);
      if(temp == 1){
       rset();
       tls_ativo=1;
       etapa_smtp=ETAPA_helo;
       strcpy(header_info_tls, ",tls");
       header_info_ini[0]='\t';
       header_info_fim[0]=')';
      }
     }else {
      // Erro no tls_init
      num_smtp_resp=MSG_internal_error;
      smtp_resp(NULL, DEF_log_lvl, NVL_log_env);
     }
     if(!tls_ativo){
      selor_superstrncpy(buffer_log, TAM_buffer_log, "(PID ", fpid, ") ERROR: tls_init/tls_accept ", tls_error(),"\n", NULL);
      log_msg(DEF_warning_log);
     }
    
    // ****************************
    // *****  NOOP / INV CMD  *****
    // ****************************
    }else {
     if(!cmp_str(smtp_cmd, "noop\0", 5)){
      num_smtp_resp=MSG_ok;
      smtp_resp(NULL, DEF_log_lvl, NVL_log_rec);
     }else {
      num_smtp_resp=MSG_invalid_cmd;
      smtp_resp(NULL, DEF_log_lvl, NVL_log_rec_env);
     }
    }
   }while(manter_conectado);
   close(socket_remoto);
   // -- Tratando arquivo(s) de fila --
   if(cnx_int){
    // Conexao Interrompida
    Cnx_interrompida:
    if(!R_CLOSE){
     if(temp1 < 0)selor_superstrncpy(buffer_log, TAM_buffer_log, "(PID ", fpid, ") ", host.ip, " - Connection timeout\n", NULL);
     else selor_superstrncpy(buffer_log, TAM_buffer_log, "(PID ", fpid, ") ", host.ip, " - Connection broken\n", NULL);
     log_msg(DEF_central_log);
    }
   }
   Proc_ids:
   if(DEF_immed_delivery == 1){
    for(temp1=msg_cont-1; temp1>=0; temp1--){          // Uso a var temp1 por ser signed.
                                                       // Comeco pelo ultimo ID para aprovietar os dados de rcpt, from, etc...
     selor_superstrncpy(id_temp, sizeof(id_temp)-1, id_pid, itoa(temp1, 0), NULL);
     proc_id(id_temp);
    }
   }else if(DEF_immed_delivery == 2 && msg_cont){
    Immed2:
    selor_superstrncpy(id_temp, sizeof(id_temp)-1, id_pid, itoa(msg_cont-1, 0), NULL);
    proc_id(id_temp);
   }
   for(cont1=0; cont1<num_mods; cont1++)if(modulo[cont1])dlclose(modulo[cont1]);
   if(to)free(to);
   return 0;
  }else if(temp1 < 0){
   // -- Erro no FORK
   selor_superstrncpy(buffer_log, TAM_buffer_log, "ERROR: Fork error (", strerror(errno), ")\n", NULL);
   falha_interna(0);   // Avisa o host de Falha interna e faz LOG
  }else {
   // -- Processo Pai
   Fechar_socket_remoto:
   close(socket_remoto);
  }
 }
 
 // Nunca vai chegar aqui
 return 0;
}


//###############################################
//#              Funcoes LOCAIS                 #
//###############################################

// --
// Retornos:
// 0 = Nao encontrado
// 1 = Encontrado    
char proc_rcpt(char *ptr_email, char responder){
 int cont1;
 
 for(cont1=0; cont1<num_rcpt; cont1++){
  if(!strcmp(to[cont1].email, ptr_email)){
   if(responder){
    num_smtp_resp=MSG_ok;
    smtp_resp(NULL, DEF_log_lvl, NVL_log_rec_env);
   }
   return 1;
  }
 }
 return 0;
}
// --
void rset(void){

 from.dominio=NULL;
 from.local=0;
 etapa_smtp=ETAPA_mail_from;
 num_rcpt=R_HITS=R_EXEC=R_SO_MARK=R_SPF=tam_header=0;
 DEF_immed_delivery=DEF_immed_delivery_bk;
 R_DATA_REPLY[0]=R_EXEC_STR[0]=R_HADD[0]=R_DSPOOL[0]=0;
 srs_from[0]=0;
 id_fila[0]=0;   // --> Caso se inicie uma nova sessao que fracasse, Isto forca o proc_id reler o arquivo de rcpts
 //R_MARK[0]=0;
}

// --
void sinal_sigterm(int sinal){
 int cont1;
 
 strcpy(buffer_log, "Server shutdown\n");
 log_msg(DEF_central_log);
 remove(DEF_pid_file);
 for(cont1=0; cont1<num_mods; cont1++){
  if(modulo[cont1]){
   mod_shutdown[cont1]();
   dlclose(modulo[cont1]);
  }
 }
 if(to)free(to);
 if(qproc)kill(qproc, SIGTERM);
 exit(0);
}

// --
void falha_interna(char finalizar){

 log_msg(DEF_warning_log);
 num_smtp_resp=MSG_internal_error;
 smtp_resp(NULL, DEF_log_lvl, NVL_log_rec_env);
 close(socket_remoto);
 if(to)free(to);
 if(finalizar)exit(1);
}

// --
#ifndef FreeBSD
 #ifndef NoCAP
  void selor_setuid(uid_t local_uid){
   struct __user_cap_header_struct header;
   struct __user_cap_data_struct capab;
   
   if(!local_uid)return;
   header.version = _LINUX_CAPABILITY_VERSION;
   header.pid = 0;
   capab.inheritable = 0;
   capab.effective=capab.permitted=1<<CAP_NET_ADMIN;
   prctl(PR_SET_KEEPCAPS, 1, 0, 0, 0);
   setuid(local_uid);
   capset(&header, &capab);
  }
 #else
  void selor_setuid(uid_t local_uid){
  
   setuid(local_uid);
  }
 #endif
#else
 void selor_setuid(uid_t local_uid){
  
  setuid(local_uid);
 }
#endif

// --
void *mod_refresh(void *parametro){
 int cont1=0;
 
 while(1){
  sleep(MOD_refresh_sleep);
  for(cont1=0; cont1<num_mods; cont1++)if(mod_cnx_refresh[cont1])mod_cnx_refresh[cont1]();
 }
}




