//======================================================
// Funcao para gerar HASH MD5                           
//                                                      
// Estas funcoes foram pegas da Wikipedia e adaptada:   
// (http://en.wikipedia.org/wiki/Md5#Algorithm)         
// Acredito que sejam de domnio publico                 
//                                                      
//======================================================
#include<stdint.h>
#include<stdio.h>
#include<string.h>
#include<stdlib.h>

// Constante de rotacao de bits
#define LEFTROTATE(x, c) (((x) << (c)) | ((x) >> (32 - (c))))
 
// Constantes senos precalculadas
const uint32_t k[64] = {0xd76aa478, 0xe8c7b756, 0x242070db, 0xc1bdceee,
                        0xf57c0faf, 0x4787c62a, 0xa8304613, 0xfd469501,
                        0x698098d8, 0x8b44f7af, 0xffff5bb1, 0x895cd7be,
                        0x6b901122, 0xfd987193, 0xa679438e, 0x49b40821,
                        0xf61e2562, 0xc040b340, 0x265e5a51, 0xe9b6c7aa,
                        0xd62f105d, 0x02441453, 0xd8a1e681, 0xe7d3fbc8,
                        0x21e1cde6, 0xc33707d6, 0xf4d50d87, 0x455a14ed,
                        0xa9e3e905, 0xfcefa3f8, 0x676f02d9, 0x8d2a4c8a,
                        0xfffa3942, 0x8771f681, 0x6d9d6122, 0xfde5380c,
                        0xa4beea44, 0x4bdecfa9, 0xf6bb4b60, 0xbebfbc70,
                        0x289b7ec6, 0xeaa127fa, 0xd4ef3085, 0x04881d05,
                        0xd9d4d039, 0xe6db99e5, 0x1fa27cf8, 0xc4ac5665,
                        0xf4292244, 0x432aff97, 0xab9423a7, 0xfc93a039,
                        0x655b59c3, 0x8f0ccc92, 0xffeff47d, 0x85845dd1,
                        0x6fa87e4f, 0xfe2ce6e0, 0xa3014314, 0x4e0811a1,
                        0xf7537e82, 0xbd3af235, 0x2ad7d2bb, 0xeb86d391};
                         
const uint32_t r[] = {7, 12, 17, 22, 7, 12, 17, 22, 7, 12, 17, 22, 7, 12, 17, 22,
                      5,  9, 14, 20, 5,  9, 14, 20, 5,  9, 14, 20, 5,  9, 14, 20,
                      4, 11, 16, 23, 4, 11, 16, 23, 4, 11, 16, 23, 4, 11, 16, 23,
                      6, 10, 15, 21, 6, 10, 15, 21, 6, 10, 15, 21, 6, 10, 15, 21};
 

// ------------------------------------------
void to_bytes(uint32_t val, uint8_t *bytes){

 bytes[0]=(uint8_t)val;
 bytes[1]=(uint8_t)(val>>8);
 bytes[2]=(uint8_t)(val>>16);
 bytes[3]=(uint8_t)(val>>24);
}

// ------------------------------------------ 
uint32_t to_int32(uint8_t *bytes){

 return (uint32_t)bytes[0] | ((uint32_t)bytes[1]<<8) | ((uint32_t)bytes[2]<<16) | ((uint32_t)bytes[3]<<24);
}

// ------------------------------------------ 
void md5(char *buffer_ini, unsigned int tam_buffer, char *buffer_final){
 uint8_t *ptr_aux1=NULL;
 uint8_t *ptr_aux2=NULL;
 uint32_t h0;
 uint32_t h1;
 uint32_t h2;
 uint32_t h3;
 uint32_t a;
 uint32_t b;
 uint32_t c;
 uint32_t d;
 uint32_t i;
 uint32_t f;
 uint32_t g;
 uint32_t w[16];
 uint32_t temp;
 unsigned int tam_novo_buffer;
 unsigned int cont1;
 
 // Calculando o tamanho buffer de trabalho
 for(tam_novo_buffer=tam_buffer+1; tam_novo_buffer%(64) != 56; tam_novo_buffer++);

 ptr_aux1=(uint8_t*)malloc(tam_novo_buffer+8);
 memcpy(ptr_aux1, buffer_ini, tam_buffer);
 
 ptr_aux1[tam_buffer]=0x80; // Setando o ultimo bit
 ptr_aux2=&ptr_aux1[tam_buffer]+1;
 for(cont1=tam_buffer+1; cont1<tam_novo_buffer; cont1++)*ptr_aux2++=0;

 to_bytes(tam_buffer*8, ptr_aux1+tam_novo_buffer);
 to_bytes(tam_buffer>>29, ptr_aux1+tam_novo_buffer+4);

 // Inicializacao 
 h0=0x67452301;
 h1=0xefcdab89;
 h2=0x98badcfe;
 h3=0x10325476;
 
 // Processando a mensagem em pedaos de 512 bytes
 for(cont1=0; cont1<tam_novo_buffer; cont1+=64){
  for(i=0; i<16; i++)w[i]=to_int32(ptr_aux1+cont1+i*4);
  // Inicializando o bloco
  a=h0;
  b=h1;
  c=h2;
  d=h3;
  for(i=0; i<64; i++){
   if(i < 16){
    f=(b&c)|((~b)&d);
    g=i;
   }else if (i < 32){
    f=(d&b)|((~d) & c);
    g=(5*i+1)%16;
   }else if (i < 48){
    f=b^c^d;
    g=(3*i+5)%16;          
   }else {
    f=c^(b|(~d));
    g=(7*i)%16;
   }
   temp=d;
   d=c;
   c=b;
   b=b+LEFTROTATE((a+f+k[i]+w[g]), r[i]);
   a=temp;
  }
  h0+=a;
  h1+=b;
  h2+=c;
  h3+=d;
 }
 free(ptr_aux1);
 to_bytes(h0, (unsigned char *)buffer_final);
 to_bytes(h1, (unsigned char *)buffer_final+4);
 to_bytes(h2, (unsigned char *)buffer_final+8);
 to_bytes(h3, (unsigned char *)buffer_final+12);
}

